class puppet_enterprise::profile::console (
  $ca_host = $puppet_enterprise::certificate_authority_host,
  $certname = $::clientcert,
  $database_host = $puppet_enterprise::database_host,
  $database_port = $puppet_enterprise::database_port,
  $database_properties = $puppet_enterprise::database_properties,
  $master_host     = $puppet_enterprise::puppet_master_host,
  $master_certname = $puppet_enterprise::puppet_master_host,
  $puppetdb_host = $puppet_enterprise::puppetdb_host,
  $puppetdb_port = $puppet_enterprise::puppetdb_port,
  $secret_token = '',
  $listen_address          = $puppet_enterprise::params::plaintext_address,
  $ssl_listen_address      = $puppet_enterprise::params::ssl_address,
  $dashboard_listen_port       = $puppet_enterprise::params::dashboard_listen_port,
  $dashboard_ssl_listen_port   = $puppet_enterprise::params::dashboard_ssl_listen_port,
  $dashboard_database_name     = $puppet_enterprise::dashboard_database_name,
  $dashboard_database_user     = $puppet_enterprise::dashboard_database_user,
  $dashboard_database_password = $puppet_enterprise::dashboard_database_password,
  $console_ssl_listen_port     = $puppet_enterprise::console_port,
  $console_services_listen_port         = $puppet_enterprise::params::console_services_listen_port,
  $console_services_ssl_listen_port     = $puppet_enterprise::params::console_services_ssl_listen_port,
  $console_services_api_listen_port     = $puppet_enterprise::params::console_services_api_listen_port,
  $console_services_api_ssl_listen_port = $puppet_enterprise::api_port,
  $activity_url_prefix          = $puppet_enterprise::params::activity_url_prefix,
  $activity_database_name       = $puppet_enterprise::activity_database_name,
  $activity_database_user       = $puppet_enterprise::activity_database_username,
  $activity_database_password   = $puppet_enterprise::activity_database_password,
  $classifier_database_name       = $puppet_enterprise::classifier_database_name,
  $classifier_database_user       = $puppet_enterprise::classifier_database_user,
  $classifier_database_password   = $puppet_enterprise::classifier_database_password,
  $classifier_url_prefix          = $puppet_enterprise::params::classifier_url_prefix,
  $rbac_database_name       = $puppet_enterprise::rbac_database_name,
  $rbac_database_user       = $puppet_enterprise::rbac_database_username,
  $rbac_database_password   = $puppet_enterprise::rbac_database_password,
  $rbac_url_prefix          = $puppet_enterprise::params::rbac_url_prefix,
  $rbac_password_reset_expiration = undef,
  $rbac_session_timeout           = undef,
  $rbac_failed_attempts_lockout   = undef,
  $localcacert = $puppet_enterprise::params::localcacert,
  $hostcrl     = $puppet_enterprise::params::hostcrl,
  $delayed_job_workers = 2,
  $disable_live_management = false,
  $migrate_db     = false,
  $whitelisted_certnames = [],
  $java_args = $puppet_enterprise::params::console_services_java_args,
  $browser_ssl_cert        = undef,
  $browser_ssl_private_key = undef,
  $browser_ssl_cert_chain  = undef,
  $browser_ssl_ca_cert     = undef,
) inherits puppet_enterprise {

  $classifier_client_certname = $puppet_enterprise::params::classifier_client_certname
  $console_client_certname = $puppet_enterprise::params::console_client_certname
  $console_server_certname = $certname

  class { 'puppet_enterprise::profile::console::certs':
    certname    => $certname,
    localcacert => $localcacert,
    hostcrl     => $hostcrl,
  }

  include puppet_enterprise::packages
  Package <| tag == 'pe-console-packages' |> {
    before => Class['puppet_enterprise::profile::console::console_services_config'],
  }

  # Unfortunately because we have no HOCON module
  # we have to keep the webserver config at the profile level.
  # When a HOCON module exists this class will consist of entries in
  # console-services' webserver/global confs for the apis (nc, rbac, activity).
  class { 'puppet_enterprise::profile::console::console_services_config':
    certname                      => $certname,
    listen_address                => $listen_address,
    listen_port                   => $console_services_listen_port,
    ssl_listen_address            => $ssl_listen_address,
    ssl_listen_port               => $console_services_ssl_listen_port,
    api_listen_port               => $console_services_api_listen_port,
    api_ssl_listen_port           => $console_services_api_ssl_listen_port,
    localcacert                   => $localcacert,
    classifier_url_prefix         => $classifier_url_prefix,
    activity_url_prefix           => $activity_url_prefix,
    rbac_url_prefix               => $rbac_url_prefix,
    notify                        => Service[ 'pe-console-services' ],
  }

  puppet_enterprise::trapperkeeper::activity { 'console-services' :
    database_host       => $database_host,
    database_port       => $database_port,
    database_name       => $activity_database_name,
    database_user       => $activity_database_user,
    database_password   => $activity_database_password,
    database_properties => $database_properties,
    rbac_host           => 'localhost',
    rbac_port           => $console_services_api_listen_port,
    rbac_url_prefix     => $rbac_url_prefix,
  }

  puppet_enterprise::trapperkeeper::rbac { 'console-services' :
    certname                  => $certname,
    database_host             => $database_host,
    database_port             => $database_port,
    database_name             => $rbac_database_name,
    database_user             => $rbac_database_user,
    database_password         => $rbac_database_password,
    database_properties       => $database_properties,
    password_reset_expiration => $rbac_password_reset_expiration,
    session_timeout           => $rbac_session_timeout,
    failed_attempts_lockout   => $rbac_failed_attempts_lockout,
  }

  file { '/etc/puppetlabs/console-services/rbac-certificate-whitelist':
    ensure => file,
    group  => 'pe-console-services',
    owner  => 'pe-console-services',
    mode   => '0640',
  }

  $certs = pe_union(['pe-internal-dashboard', $master_certname, $certname], $whitelisted_certnames)
  puppet_enterprise::certs::rbac_whitelist_entry { $certs: }
  Puppet_enterprise::Certs::Rbac_whitelist_entry <<| certname != $master_certname and certname != $certname |>>

  puppet_enterprise::certs::dashboard_whitelist_entry { $certs: }
  Puppet_enterprise::Certs::Dashboard_whitelist_entry <<| certname != $master_certname and certname != $certname |>>

  puppet_enterprise::trapperkeeper::classifier { 'console-services' :
    master_host         => $master_host,
    database_host       => $database_host,
    database_port       => $database_port,
    database_name       => $classifier_database_name,
    database_user       => $classifier_database_user,
    database_password   => $classifier_database_password,
    database_properties => $database_properties,
    client_certname     => $classifier_client_certname,
    localcacert         => $localcacert,
  }

  class { 'puppet_enterprise::console_services':
    client_certname       => $console_client_certname,
    dashboard_port        => $dashboard_listen_port,
    classifier_host       => 'localhost',
    classifier_port       => $console_services_api_listen_port,
    classifier_url_prefix => $classifier_url_prefix,
    puppetdb_host         => $puppetdb_host,
    puppetdb_port         => $puppetdb_port,
    rbac_host             => 'localhost',
    rbac_port             => $console_services_api_listen_port,
    activity_host         => 'localhost',
    activity_port         => $console_services_api_listen_port,
    activity_url_prefix   => $activity_url_prefix,
    localcacert           => $localcacert,
    java_args             => $java_args,
  }

  class { 'puppet_enterprise::profile::console::apache_proxy' :
    certname                           => $certname,
    trapperkeeper_proxy_listen_address => $listen_address,
    trapperkeeper_proxy_listen_port    => $console_services_listen_port,
    ssl_listen_address                 => $ssl_listen_address,
    ssl_listen_port                    => $console_ssl_listen_port,
    browser_ssl_cert                   => $browser_ssl_cert,
    browser_ssl_private_key            => $browser_ssl_private_key,
    browser_ssl_cert_chain             => $browser_ssl_cert_chain,
    browser_ssl_ca_cert                => $browser_ssl_ca_cert,
  }

  class { 'puppet_enterprise::console':
    database_host           => $database_host,
    database_port           => $database_port,
    master_host             => $master_host,
    localcacert             => $localcacert,
    hostcrl                 => $hostcrl,
    console_server_certname => $console_server_certname,
    console_client_certname => $console_client_certname,
    ca_host                 => $ca_host,
    puppetdb_host           => $puppetdb_host,
    puppetdb_port           => $puppetdb_port,
    listen_address          => $listen_address,
    listen_port             => $dashboard_listen_port,
    ssl_listen_address      => $ssl_listen_address,
    ssl_listen_port         => $dashboard_ssl_listen_port,
    secret_token            => $secret_token,
    database_name           => $dashboard_database_name,
    database_user           => $dashboard_database_user,
    database_password       => $dashboard_database_password,
    migrate_db              => $migrate_db,
    classifier_host         => $console_server_certname,
    classifier_port         => $console_services_api_ssl_listen_port,
    rbac_host               => $console_server_certname,
    rbac_port               => $console_services_api_ssl_listen_port,
    certname                => $certname,
    delayed_job_workers     => $delayed_job_workers,
    disable_live_management => $disable_live_management,
  }
}
